###############################################################################
###############################################################################
##                        Herramientas.tcl
###############################################################################
###############################################################################
## Includes procedures to implement the commands in the 'Tools' and 'Options'
## menus.
###############################################################################
###############################################################################
## (c) 1999-2008 Andres Garcia Garcia. fandom@telefonica.net
## Distributed under the terms of the GPL v2
###############################################################################
###############################################################################

namespace eval Herramientas {

###############################################################################
# SaveConfig
#   Saves the configuration
###############################################################################
proc SaveConfig {} {
    global urlHistory urlIndex dirGetleft getleftOptions urlsToDownload
    global getleftState defaultDownOptions env
    global labelMessages labelTitles

    if {$getleftState(os)=="win"} {
        set ficherorc [file join "$dirGetleft(conf)" getleft.ini]
        # I got a bug report that said that Win2K doesn't like to
        # open hidden files, so I unhide it before opening
        if {[file exists $ficherorc]} {
            file attributes $ficherorc -hidden 0
        }
    } else {
        set ficherorc [file join "$dirGetleft(conf)" getleft.conf]
    }
    if {[catch {open  $ficherorc w+} fichero]} {
        Dialogos::Dialogo .error -title $labelTitles(error) -icon error \
            -message "$labelMessages(saveConfig)\n$fichero"
        return
    }

    puts $fichero "# Please do not edit unless you know what you are doing.\n\n"

    if {![info exists ::wizFull]} {
        puts $fichero "\[urlHistory\]\n"
        if {![catch {llength $urlHistory} len]} {
            for {set i 0} {$i<$len} {incr i} {
                puts $fichero [lindex $urlHistory $i]
            }
        }
        puts $fichero "\n\[urlIndex\]\n"
        puts $fichero $urlIndex
    }

    if {[info exists getleftOptions(httpProxy)]} {
        puts $fichero "\n\[httpProxy\]\n"
        puts $fichero "$getleftOptions(httpProxy)"
        puts $fichero "\n\[ftpProxy\]\n"
        puts $fichero "$getleftOptions(ftpProxy)"
        puts $fichero "\n\[sameProxy\]\n"
        puts $fichero "$getleftOptions(sameProxy)"
        if {$getleftOptions(useAuthProxy)==1} {
            if {$getleftOptions(saveAuthPass)==1} {
                puts $fichero "\n\[proxyPass\]\n"
                puts $fichero "$getleftOptions(proxyPass)"
            }
            puts $fichero "\n\[proxyUser\]\n"
            puts $fichero "$getleftOptions(proxyUser)"
            puts $fichero "\n\[useAuthProxy\]\n"
            puts $fichero "$getleftOptions(useAuthProxy)"
            puts $fichero "\n\[saveAuthPass\]\n"
            puts $fichero "$getleftOptions(saveAuthPass)"
        }
    }

    puts $fichero "\n\[language\]\n"
    puts $fichero "$getleftOptions(lang)"

    if {[info exists getleftState(browser)]} {
        puts $fichero "\n\[browser\]\n"
        puts $fichero "$getleftState(browser)"
    }

    if {![info exists ::wizFull]} {
        puts $fichero "\n\[lastDir\]\n"
        puts $fichero "$dirGetleft(lastDir)"

        puts $fichero "\n\[toolDir\]\n"
        puts $fichero "$dirGetleft(toolDir)"

        puts $fichero "\n\[delaySet\]\n"
        puts $fichero "$getleftState(delaySet)"

        puts $fichero "\n\[defaultOptions\]\n"
        puts $fichero [array get defaultDownOptions]

        if {$getleftState(os) eq "unix"} {
            puts $fichero "\n\[theme\]\n"
            puts $fichero $getleftState(theme)
        }

        if {$getleftState(delaySet)==1} {
            puts $fichero "\n\[delayData\]\n"
            puts $fichero "array set Delay::delayedDownload \
                          \{[array get Delay::delayedDownload]\}"
        }

        if {[llength $getleftState(urlQueue)]} {
            puts $fichero "\n\[urlQueue\]\n"
            foreach site $getleftState(urlQueue) {
                puts $fichero $site
            }
            puts $fichero "\n\[urlsToDownload\]\n"
            foreach site $getleftState(urlQueue) {
                regsub -all {\s} $site    {\\&} siteTmp
                regsub -all {\;} $siteTmp {\\;} siteTmp
                puts $fichero "set urlsToDownload($siteTmp)         1"
                puts $fichero "set urlsToDownload($siteTmp,dir)     \"$urlsToDownload($site,dir)\""
                puts $fichero "set urlsToDownload($siteTmp,resume)  $urlsToDownload($site,resume)"
                puts $fichero "set urlsToDownload($siteTmp,options) [list $urlsToDownload($site,options)]"
            }
        }
    }

    close $fichero

    if {$getleftState(os) eq "win"} {
        file attributes $ficherorc -hidden 1
    } else {
        file attributes $ficherorc -permissions 0600
    }

    return
}

###############################################################################
# GetMainDirectory
#    This procedure investigates where the 'Getleft.tcl' file is.
#
# Returns:
#    The directory.
###############################################################################
proc GetMainDirectory {} {

    set scriptDir [info script]
    if {[file pathtype $scriptDir]=="relative"} {
        set scriptDir [string trimleft $scriptDir ./]
        set scriptDir [file join [pwd] $scriptDir]
    }
    return [file dirname $scriptDir]
}

###############################################################################
# SetGetleftDirs
#   A simplified version of the procedure of the same name to be used
#   exclusively from the 'install.tcl' script.
###############################################################################
proc SetGetleftDirs {} {
    global dirGetleft env getleftState

    set dirGetleft(main)    [GetMainDirectory]
    set dirGetleft(lastDir) $env(HOME)
    set dirGetleft(toolDir) $env(HOME)
    set dirGetleft(conf)    [file join $env(HOME) .getleft]

    if {![file exists $dirGetleft(conf)]} {
        if {[catch {file mkdir $dirGetleft(conf)} error]} {
            Dialogos::Dialogo .error -title Error -type ok   \
                    -message "Couldn't create dir:\n$error"
            exit
        }
    }

    set dirGetleft(scripts)       [file join "$dirGetleft(main)" scripts]
    if {[file exists $dirGetleft(scripts)]} {
        set dirGetleft(images)    [file join "$dirGetleft(main)" images]
        set dirGetleft(doc)       [file join "$dirGetleft(main)" doc]
        set dirGetleft(languages) [file join "$dirGetleft(main)" Languages]
    }

    set dirGetleft(icons) $dirGetleft(images)/Crystal

    return
}

###############################################################################
# CreateShortcut
#    Uses the Portland utilities to create either a menu entry or a desktop
#    shortcut
#
# Parameter
#    which: 'menu' or 'desktop'
###############################################################################
proc CreateShortcut {which} {
    global dirGetleft env labelTitles
    variable installedIcons
    puts "Creamos: $which"

    set path $env(PATH)
    # First we install the icons
    set iconsDir [file join $dirGetleft(images) Shortcuts]
    set xdgDir   [file join $iconsDir xdg]

    if {![regexp "$xdgDir" $path]} {
        set env(PATH) $path:$xdgDir
    }
    if {![info exists installedIcons]} {
        foreach size {16 22 32 48 64 128} {
            set iconFile [file join $iconsDir Getleft${size}x${size}.png]
            if {[catch {exec xdg-icon-resource install --novendor --size $size \
                   --context apps $iconFile getleft} error]} {
                Dialogos::Dialogo .error -type ok -icon error                  \
                        -title $labelTitles(error)                             \
                        -message "$error"
            }
        }
        set installedIcons 1
    }
    # We adapt the desktop templete file
    set desktopFile [file join $iconsDir Getleft.desktop]
    file copy -force $desktopFile.tpl $desktopFile
    set    handle [open "$desktopFile" a]
    puts  $handle "Exec = [file join $dirGetleft(main) Getleft.tcl]"
    close $handle

    if {$which eq "desktop"} {
        if {[catch {exec xdg-desktop-icon install --novendor $desktopFile} error]} {
            Dialogos::Dialogo .error -type ok -icon error                 \
                    -title $labelTitles(error)                            \
                    -message "$error"
        }
        return
    }
    if {[catch {exec xdg-desktop-menu install --novendor $desktopFile} error]} {
        Dialogos::Dialogo .error -type ok -icon error                 \
                -title $labelTitles(error)                            \
                -message "$error"
    }
    return
}

###############################################################################
# ExtensionDialog
#    Creates a dialog box to choose file extensions
#
# Parameters:
#    title: title of the dialog box
#    parent: the window over which the dialog will be created.
#    filter: a string with the current filter, defaults to empty.
#
# Side effect:
#    Namespace scope variable 'bot' will contain the choosen extensions
#
# Returns:
#    1: if cancelled
###############################################################################
proc ExtensionDialog {title parent {filter ""}} {
    global labelButtons labelTitles downOptions indexButtons getleftState
    variable done

    set coord(x) [winfo rootx $parent]
    set coord(y) [winfo rooty $parent]

    set win [toplevel .extDialog]
    wm title $win $title
    wm resizable $win 0 0
    wm geometry $win +[expr {$coord(x)+100}]+[expr {$coord(y)+15}]
    keynav::enableMnemonics $win

    set marcoExt [ttk::frame $win.marcoext]
    set marcoInt [ttk::frame $marcoExt.marcoInt -relief groove]
    set marcoBot [ttk::frame $marcoExt.marcoBot]

    set izqBot1  [ttk::checkbutton $marcoInt.bot1  -variable ::Herramientas::bot(1) \
            -onvalue "jpg" -offvalue "" -text "*.jpg"]
    set izqBot2  [ttk::checkbutton $marcoInt.bot2  -variable ::Herramientas::bot(2) \
            -onvalue "gif" -offvalue "" -text "*.gif"]
    set izqBot3  [ttk::checkbutton $marcoInt.bot3  -variable ::Herramientas::bot(3) \
            -onvalue "mp3" -offvalue "" -text "*.mp3"]
    set izqBot4  [ttk::checkbutton $marcoInt.bot4  -variable ::Herramientas::bot(4) \
            -onvalue "ps"  -offvalue "" -text "*.ps" ]

    set cenBot1  [ttk::checkbutton $marcoInt.bot5  -variable ::Herramientas::bot(5) \
            -onvalue "pdf" -offvalue "" -text "*.pdf"]
    set cenBot2  [ttk::checkbutton $marcoInt.bot6  -variable ::Herramientas::bot(6) \
            -onvalue "txt" -offvalue "" -text "*.txt"]
    set cenBot3  [ttk::checkbutton $marcoInt.bot7  -variable ::Herramientas::bot(7) \
            -onvalue "mov" -offvalue "" -text "*.mov"]
    set cenBot4  [ttk::checkbutton $marcoInt.bot8  -variable ::Herramientas::bot(8) \
            -onvalue "mpg" -offvalue "" -text "*.mpg"]

    set derBot1  [ttk::checkbutton $marcoInt.bot9  -variable ::Herramientas::bot(9) \
            -onvalue "avi" -offvalue "" -text "*.avi"]
    set derBot2  [ttk::checkbutton $marcoInt.bot10 -variable ::Herramientas::bot(10)\
            -onvalue "zip" -offvalue "" -text "*.zip"]
    set derBot3  [ttk::checkbutton $marcoInt.bot11 -variable ::Herramientas::bot(11)\
            -onvalue "tgz" -offvalue "" -text "*.tgz"]
    set derBot4  [ttk::checkbutton $marcoInt.bot12 -variable ::Herramientas::bot(12)\
            -onvalue "sit" -offvalue "" -text "*.sit"]


    if {![string match $title $labelTitles(exclude)]} {
        foreach {index ext} {1 jpg 2 gif 3 mp3 4 ps 5 pdf 6 txt 7 mov 8 mpg 9 avi 10 zip 11 tgz 12 sit} {
            set ::Herramientas::bot($index) $ext
        }
    } else {
        foreach {index ext} {1 jpg 2 gif 3 mp3 4 ps 5 pdf 6 txt 7 mov 8 mpg 9 avi 10 zip 11 tgz 12 sit} {
            if {[regexp "$ext" $filter]} {
                set ::Herramientas::bot($index) $ext
            } else {
                set ::Herramientas::bot($index) ""
            }
        }
    }

    set aceptar  [ttk::button $marcoBot.aceptar                             \
        -textvariable labelButtons(ok)     -underline $indexButtons(ok)     \
        -width 8 -command "set ::Herramientas::done 1"]
    set cancelar [ttk::button $marcoBot.cancelar                            \
        -textvariable labelButtons(cancel) -underline $indexButtons(cancel) \
        -width 8 -command "set ::Herramientas::done 0"]

    bind $win <Escape> "$cancelar invoke"

    pack $marcoExt -ipadx 10 -ipady  5
    pack $marcoBot -fill x -padx 7 -side bottom
    pack $marcoInt -ipadx 10 -ipady 15 -side bottom

    if {$::tcl_version>=8.5} {
        grid anchor $marcoInt   c
    }

    grid $izqBot1 $cenBot1 $derBot1 -sticky w -padx 3
    grid $izqBot2 $cenBot2 $derBot2 -sticky w -padx 3
    grid $izqBot3 $cenBot3 $derBot3 -sticky w -padx 3
    grid $izqBot4 $cenBot4 $derBot4 -sticky w -padx 3

    if {$getleftState(os) ne "mac"} {
        pack $cancelar $aceptar  -side right -pady 5 -padx 3
    } else {
        pack $aceptar  $cancelar -side right -pady 5 -padx 3
    }
    tkwait variable ::Herramientas::done
    destroy $win
    if {$::Herramientas::done==0} {
        return 1
    }
    return
}

###############################################################################
# PurgeFilesDir
#    Sweeps a directory turning files to size 0
#
# Parameters:
#    dir: directory to be processed
#    purgarCmd: extensions of the files to minimize
###############################################################################
proc PurgeFilesDir {dir purgarCmd} {
    global dirGetleft

    cd $dir
    set archivos ""
    set extList  [concat $purgarCmd ram jpeg zip aiff]
    foreach ext $extList {
        set archivos [concat $archivos [glob -nocomplain *.$ext]]
        set archivos [concat $archivos [glob -nocomplain [string toupper *.$ext]]]
    }

    foreach file $archivos {
        file stat $file estado
        if {$estado(size)!=0} {
            file delete -force $file
            Commands::Touch $file
        }
    }

    set directorios [glob -nocomplain */]

    foreach dir $directorios {
        update
        PurgeFilesDir $dir $purgarCmd
        cd ..
    }
    return
}

###############################################################################
# PurgeFiles
#    The procedures takes care of recursively clean directories of files
#    with certain extensions *.jpg, *.gif, etc.
###############################################################################
proc PurgeFiles {} {
    global labelTitles labelMessages dirGetleft

    if {[winfo exists .extDialog]} {
        raise .extDialog .
        return
    }

    if {[ExtensionDialog $labelTitles(purge) .]==1} {
        return
    }

    set dir [Dialogos::SelectDirectory $dirGetleft(toolDir)]
    if {![string compare $dir ""]} {
        return
    }
    set dirGetleft(toolDir) [file dirname $dir]

    set dirTmp [pwd]
    for {set i 1;set purgarCmd ""} {$i<13} {incr i} {
        if {$::Herramientas::bot($i)=="tgz"} {
            append purgarCmd tar.gz " "
        } elseif {$::Herramientas::bot($i)=="jpg"} {
            append purgarCmd jpeg   " "
        } elseif {$::Herramientas::bot($i)=="mpg"} {
            append purgarCmd mpeg   " "
        }
        append purgarCmd $::Herramientas::bot($i) " "
    }
    PurgeFilesDir $dir $purgarCmd

    Dialogos::Dialogo .info -title $labelTitles(theEnd) -icon info           \
        -message $labelMessages(purged)

    cd $dirTmp

    return
}

###############################################################################
# RestoreOriginalsDir
#    Recursively restores the original files.
#
# Parameter:
#    dir: directory in which it will begin to restore.
###############################################################################
proc RestoreOriginalsDir {dir} {

    cd $dir
    set directorio [glob -nocomplain *.orig]
    foreach fichero $directorio {
        file rename -force -- $fichero [file root $fichero]
    }

    set directories [glob -nocomplain */]
    foreach dir $directories {
        RestoreOriginalsDir $dir
        cd ..
    }
    return
}

###############################################################################
# RestoreOriginals
#    Starts the process of restoring the original files downloaded from Web
#    pages.
###############################################################################
proc RestoreOriginals {} {
    global labelTitles labelMessages dirGetleft

    set dir [Dialogos::SelectDirectory $dirGetleft(toolDir)]
    if {![string compare $dir ""]} {
        return
    }
    set dirGetleft(toolDir) [file dirname $dir]
    set dirTmp [pwd]
    RestoreOriginalsDir $dir

    Dialogos::Dialogo .info -title $labelTitles(theEnd)       \
            -icon info -message $labelMessages(restored)

    cd $dirTmp

    return
}

###############################################################################
# SameProxy
#    Disables or enables the entries for the ftp proxy, depending on
#    the checkbutton
###############################################################################
proc SameProxy {} {
    global getleftOptions
    variable ip

    if {![info exists getleftOptions(sameProxy)]} {
        set getleftOptions(sameProxy) 0
    }

    for {set i 3} {$i<5} {incr i} {
        if {$getleftOptions(sameProxy)==1} {
            $ip($i) configure -state disabled
        } else {
            $ip($i) configure -state normal
        }
    }

    return
}

###############################################################################
# AuthenProxy
#    Disables or enables the entries for authentificatinf proxies.
###############################################################################
proc AuthenProxy {} {
    global getleftOptions
    variable authen

    if {![info exists getleftOptions(useAuthProxy)]} {
        set getleftOptions(useAuthProxy) 0
    }

    if {$getleftOptions(useAuthProxy)==1} {
        $authen(name) configure -state normal
        if {$getleftOptions(saveAuthPass)==1} {
            $authen(pass) configure -state normal
        } else {
            $authen(pass) configure -state disabled
        }
        $authen(save) configure -state normal
    } else {
        $authen(name) configure -state disabled
        $authen(pass) configure -state disabled
        set getleftOptions(saveAuthPass) 0
        $authen(save) configure -state disabled
    }

    return
}

###############################################################################
# ConfProxyControl
#    Takes the action ordered by the user in the proxy dialog.
#
# Parameter
#    action: 'clear' if the user wants to clear the configuration.
#            'accept' if the user accepts it.
#            'cancel' the user cancels the dialog
#    parent: parent widget of the conf dialog.
#
# Returns:
#    '0' if everything went well and '1' if there was an error.
###############################################################################
proc ConfProxyControl {action {parent .proxy}} {
    global getleftOptions labelMessages labelTitles
    variable ip
    variable getleftOptionsTemp
    variable authen

    switch -exact -- $action {
        clear {
            set getleftOptions(sameProxy) 0
            SameProxy
            for {set i 1} {$i<5} {incr i} {
                $ip($i) delete 0 end
            }
            catch {unset getleftOptions(httpProxy)}
            catch {unset getleftOptions(ftpProxy)}
            set getleftOptions(proxy) 0
            $authen(name) delete 0 end
            $authen(pass) delete 0 end
            catch {unset getleftOptions(proxyUser)}
            catch {unset getleftOptions(proxyPass)}
            set getleftOptions(useAuthProxy) 0
            set getleftOptions(saveAuthPass) 0
            AuthenProxy
            return 0
        }
        accept {
            for {set i 1} {$i<5} {incr i} {
                set tmp [$ip($i) get]
                set dirProxy($i) $tmp
                if {($getleftOptions(sameProxy)==1)&&($i==2)} break
            }
            if {($dirProxy(1)=="")&&($dirProxy(2)=="")} {
                catch {unset getleftOptions(httpProxy)}
                catch {unset getleftOptions(ftpProxy)}
            } else {
                set getleftOptions(httpProxy)    "$dirProxy(1):$dirProxy(2)"
                if {$getleftOptions(sameProxy)==0} {
                    set getleftOptions(ftpProxy) "$dirProxy(3):$dirProxy(4)"
                } else {
                    set getleftOptions(ftpProxy) $getleftOptions(httpProxy)
                }
                set getleftOptions(proxy)     1
                set getleftOptions(proxyUser) [$authen(name) get]
                set getleftOptions(proxyPass) [$authen(pass) get]
                if {($getleftOptions(useAuthProxy)==1)\
                    &&($getleftOptions(proxyUser)=="")} {
                    Dialogos::Dialogo $parent.error -type ok -icon error                \
                            -message $labelMessages(noUser) -title $labelTitles(error)
                    return 1
                }
                if {($getleftOptions(saveAuthPass)==1)                                  \
                        &&($getleftOptions(proxyPass)=="")} {
                    Dialogos::Dialogo $parent.error -type ok -icon error                \
                            -message $labelMessages(noPass) -title $labelTitles(error)
                    return 1
                }
            }
        }
        cancel {
            array set getleftOptions [array get getleftOptionsTemp]
        }
    }

    if {![info exists ::wizFull]} {
        SaveConfig
    }
    destroy .proxy

    return 0
}

###############################################################################
# ConfProxyWindowCommon
#    This procedure takes care of creating the parts of the window that are
#    shared between the proper 'Proxy configure' dialog and the one in the
#    configuration wizard.
#
# Parameter:
#    The widget in which it will be put.
###############################################################################
proc ConfProxyWindowCommon {parent} {
    global getleftOptions indexDialogs
    variable ip
    variable authen

    set marcoEx   [ttk::frame $parent.marcoEx]
    set marcoIn   [ttk::frame $marcoEx.marcoIn]
    set marcoIp   [ttk::labelframe $marcoIn.marcoIp   -labelanchor nw               \
            -labelwidget [ttk::label $marcoIn.marcoIplb                             \
            -textvariable  labelFrames(proxy)]]
    raise $marcoIn.marcoIplb

    set label1 [ttk::label $marcoIp.label1 -textvariable labelDialogs(http)]
    set ip(1)  [ttk::entry $marcoIp.1 -width 20 -justify  left]
    set ip(2)  [ttk::entry $marcoIp.2 -width 4]
    set colon1 [ttk::label $marcoIp.colon1 -text : -width 2]

    set label2 [ttk::label $marcoIp.label2  -textvariable labelDialogs(ftp)]
    set ip(3)  [ttk::entry $marcoIp.3 -width 20 -justify  left]
    set ip(4)  [ttk::entry $marcoIp.4 -width 4]
    set colon2 [ttk::label $marcoIp.colon2 -text : -width 2]

    set checkIp [ttk::checkbutton $marcoIp.check                                    \
            -textvariable labelDialogs(sameProxy) -underline $indexDialogs(sameProxy)\
            -variable getleftOptions(sameProxy)   -command ::Herramientas::SameProxy]
    set marcoAuth    [ttk::labelframe $marcoIn.marcoAuth -labelanchor nw            \
            -labelwidget [ttk::label $marcoIn.marcoAuthlb                           \
            -textvariable labelFrames(authen)]]
    raise $marcoIn.marcoAuthlb

    set nameLabel    [ttk::label $marcoAuth.nameLabel -textvariable labelDialogs(username)]
    set authen(name) [ttk::entry $marcoAuth.nameEntry -width 15]
    set passLabel    [ttk::label $marcoAuth.passLabel -textvariable labelDialogs(password)]
    set authen(pass) [ttk::entry $marcoAuth.passEntry -width 15 -show *]

    set authen(use)  [ttk::checkbutton $marcoAuth.use                               \
        -command Herramientas::AuthenProxy    -underline $indexDialogs(authenUse)   \
        -textvariable labelDialogs(authenUse) -variable getleftOptions(useAuthProxy)]
    set authen(save) [ttk::checkbutton $marcoAuth.save                              \
        -command Herramientas::AuthenProxy    -underline $indexDialogs(authenSave)  \
        -textvariable labelDialogs(authenSave) -variable getleftOptions(saveAuthPass)]

    if {$::tcl_version>=8.5} {
        grid anchor $marcoEx   c
        grid anchor $marcoIn   c
        grid anchor $marcoIp   c
        grid anchor $marcoAuth c
    }

    grid $marcoEx -ipadx 7   -ipady 5
    grid $marcoIn -sticky s  -pady 5
    grid $marcoIp $marcoAuth -padx 5 -ipadx 10 -ipady 7 -sticky ns
    grid $label1 -in $marcoIp -row 0 -column 0 -sticky w -pady 3
    grid $ip(1)  -in $marcoIp -row 0 -column 1 -sticky w -pady 3
    grid $colon1 -in $marcoIp -row 0 -column 2 -sticky w -pady 3
    grid $ip(2)  -in $marcoIp -row 0 -column 3 -sticky e -pady 3
    grid $label2 -in $marcoIp -row 1 -column 0 -sticky w -pady 3
    grid $ip(3)  -in $marcoIp -row 1 -column 1 -sticky w -pady 3
    grid $colon2 -in $marcoIp -row 1 -column 2 -sticky w -pady 3
    grid $ip(4)  -in $marcoIp -row 1 -column 3 -sticky e -pady 3
    grid configure $checkIp -columnspan 4

    grid $nameLabel    -in $marcoAuth -row 0 -column 0 -padx 5 -pady 3 -sticky w
    grid $authen(name) -in $marcoAuth -row 0 -column 1 -padx 5 -pady 3
    grid $passLabel    -in $marcoAuth -row 1 -column 0 -padx 5 -pady 3 -sticky w
    grid $authen(pass) -in $marcoAuth -row 1 -column 1 -padx 5 -pady 3
    grid $nameLabel    -in $marcoAuth -row 0 -column 0 -padx 5 -pady 3
    grid configure $authen(use)  -padx 5 -columnspan 2
    grid configure $authen(save) -padx 5 -columnspan 2

    focus $ip(1)
    bind $ip(1) <Return>   "focus $ip(2)"
    bind $ip(1) <KP_Enter> "focus $ip(2)"
    bind $ip(2) <Return>   "focus $ip(3)"
    bind $ip(2) <KP_Enter> "focus $ip(3)"
    bind $ip(3) <Return>   "focus $ip(4)"
    bind $ip(3) <KP_Enter> "focus $ip(4)"
    bind $authen(name) <Return>   "focus $authen(pass)"
    bind $authen(name) <KP_Enter> "focus $authen(pass)"
    bind $authen(pass) <Return>   "focus $authen(use)"
    bind $authen(pass) <KP_Enter> "focus $authen(use)"
    bind $authen(use)  <Key-Down> "focus $authen(save)"
    bind $authen(save) <Key-Up>   "focus $authen(use)"

    if {[info exists getleftOptions(httpProxy)]} {
        regexp {(.+)(?::)(.+)} $getleftOptions(httpProxy) nada name port
        $ip(1) insert insert $name
        $ip(2) insert insert $port
    }
    if {[info exists getleftOptions(ftpProxy)]} {
        regexp {(.+)(?::)(.+)} $getleftOptions(ftpProxy)  nada name port
        $ip(3)  insert insert $name
        $ip(4)  insert insert $port
    }
    SameProxy

    catch {$authen(name) insert insert $getleftOptions(proxyUser)}
    catch {$authen(pass) insert insert $getleftOptions(proxyPass)}
    AuthenProxy

    return
}

###############################################################################
# ConfProxyWindow
#    Creates the window in which the user enters his proxy data.
###############################################################################
proc ConfProxyWindow {} {
    global labelTitles indexButtons getleftState

    set coord(x) [winfo rootx .]
    set coord(y) [winfo rooty .]

    set proxy [toplevel .proxy]
    wm title $proxy $labelTitles(proxy)
    wm resizable $proxy 0 0
    wm geometry  $proxy +[expr {$coord(x)+75}]+[expr {$coord(y)+75}]
    keynav::enableMnemonics $proxy

    ConfProxyWindowCommon $proxy

    set botones [ttk::frame  $proxy.marcoEx.botones -class BottonFrame]
    set clear   [ttk::button $botones.clear                                     \
            -textvariable labelButtons(clear)  -underline $indexButtons(clear)  \
            -width 8 -command {::Herramientas::ConfProxyControl clear}]
    set accept [ttk::button $botones.accept                                     \
            -textvariable labelButtons(ok)     -underline $indexButtons(ok)     \
            -width 8 -command {::Herramientas::ConfProxyControl accept}]
    set cancel [ttk::button $botones.cancel                                     \
            -textvariable labelButtons(cancel) -underline $indexButtons(cancel) \
            -width 8 -command {::Herramientas::ConfProxyControl cancel}]

    bind $proxy <Escape> "$cancel invoke"

    grid $botones -sticky e -padx 3
    if {$getleftState(os) ne "mac"} {
        grid $clear $accept $cancel -padx 3 -sticky n
    } else {
        grid $clear $cancel $accept -padx 3 -sticky n
    }

    return
}

###############################################################################
# ConfProxy
#     Allows the user to enter the proxy data
###############################################################################
proc ConfProxy {} {
    global   getleftOptions
    variable getleftOptionsTemp

    if {[winfo exists .proxy]} {
        raise .proxy .
        return
    }

    array set getleftOptionsTemp [array get getleftOptions]
    ConfProxyWindow

    return
}

###############################################################################
# GetPassControl
#    Takes the action ordered by the user in the 'Get Password' dialog.
###############################################################################
proc GetPassControl {} {
    global getleftOptions labelMessages
    variable done
    variable authen

    set getleftOptions(proxyUser) [$authen(name) get]
    set getleftOptions(proxyPass) [$authen(pass) get]

    if {($getleftOptions(proxyPass)=="")} {
        Dialogos::Dialogo .getPass.error -type ok -icon error                \
                -message $labelMessages(noPass)
        return
    }

    SaveConfig
    set done 1

    return
}

###############################################################################
# GetPassWindow
#    Dialog to get the Username and password for the proxy.
###############################################################################
proc GetPassWindow {} {
    global getleftOptions labelButtons labelTitles labelDialogs labelFrames
    variable done
    variable authen

    set coord(x) [winfo rootx .]
    set coord(y) [winfo rooty .]

    set win [toplevel .getPass]
    wm title $win $labelTitles(proxy)
    wm resizable $win 0 0
    wm geometry  $win +[expr {$coord(x)+125}]+[expr {$coord(y)+75}]

    set done 0

    set marcoEx    [ttk::frame $win.marcoEx]
    set marcoLabel [ttk::labelframe $marcoEx.marcoIn    -labelanchor nw          \
            -labelwidget [ttk::label $marcoEx.marcoInlb                          \
            -textvariable labelFrames(authen)]]
    raise $marcoEx.marcoInlb

    set marcoIn    [ttk::frame $marcoLabel.in]
    set nameLabel    [ttk::label $marcoIn.nameLabel -textvariable labelDialogs(username)]
    set authen(name) [ttk::entry $marcoIn.nameEntry -width 15]
    set passLabel    [ttk::label $marcoIn.passLabel -textvariable labelDialogs(password)]
    set authen(pass) [ttk::entry $marcoIn.passEntry -width 15 -show *]

    menuEntry::menuEntry $authen(name)
    menuEntry::menuEntry $authen(pass)

    set authen(save) [ttk::checkbutton $marcoIn.save \
        -textvariable labelDialogs(authenSave) -variable getleftOptions(saveAuthPass)]

    set botones  [ttk::frame  $marcoEx.botones]
    set aceptar  [button $botones.aceptar  -textvariable labelButtons(ok)    \
            -width 8 -command ::Herramientas::GetPassControl]
    set cancelar [button $botones.cancelar -textvariable labelButtons(cancel)\
            -width 8 -command {set ::Herramientas::done 0}]

    grid $marcoEx -padx 7   -ipady 5
    grid $marcoLabel -padx 5 -pady 5
    grid $marcoIn -sticky s -pady 10 -padx 5
    grid $nameLabel    -in $marcoIn -row 0 -column 0 -padx 5 -pady 3 -sticky w
    grid $authen(name) -in $marcoIn -row 0 -column 1 -padx 5 -pady 3
    grid $passLabel    -in $marcoIn -row 1 -column 0 -padx 5 -pady 3 -sticky w
    grid $authen(pass) -in $marcoIn -row 1 -column 1 -padx 5 -pady 3
    grid configure $authen(save) -padx 5 -columnspan 2

    grid $botones -sticky e -padx 2
    grid $aceptar $cancelar $aceptar -padx 3 -sticky n

    bind $win          <Escape>   "$cancelar invoke"
    bind $authen(name) <Return>   "focus $authen(pass)"
    bind $authen(name) <KP_Enter> "focus $authen(pass)"
    bind $authen(pass) <Return>   "focus $authen(save)"
    bind $authen(pass) <KP_Enter> "focus $authen(save)"
    bind $authen(save) <Key-Up>   "focus $authen(save)"
    bind $authen(save) <Key-Down> "focus $aceptar"

    if {[catch {$authen(name) insert insert $getleftOptions(proxyUser)}]} {
        focus $authen(name)
    } else {
        focus $authen(pass)
    }

    return
}

###############################################################################
# GetPass
#     Queries the user for the password.
#
# Returns
#     '1' if a password is set, '0' otherwise
###############################################################################
proc GetPass {} {
    global getleftOptions

    if {[winfo exists .getPass]} {
        raise .getPass .
        return
    }

    array set getleftOptionsTemp [array get getleftOptions]
    GetPassWindow
    tkwait variable ::Herramientas::done
    if {$::Herramientas::done==0} {
        array set getleftOptions [array get getleftOptionsTemp]
    }

    destroy .getPass

    return $::Herramientas::done
}

###############################################################################
# FilterFiles
#    You can choose which file extensions will be ignored while downloading a
#    Web site.
#
# Parameter:
#    filter: The current filter.
#    parent: The path of the window over which the dialog will be shown,
#            defaults to the main window.
#
# Returns:
#    A string with the filter, if the users cancells the dialog, it will be
#    the same string passed as a parameter.
###############################################################################
proc FilterFiles {filter {parent .}} {
    global labelTitles

    if {[winfo exists .extDialog]} {
        raise .extDialog .
        return $filter
    }

    if {[ExtensionDialog $labelTitles(exclude) $parent $filter]==1} {
        return $filter
    }
    catch {unset urlsDownloaded}
    for {set i 1;set filter ""} {$i<13} {incr i} {
        if {$::Herramientas::bot($i)!=""} {
            if {$::Herramientas::bot($i)=="tgz"} {
                append filter (tar.gz$) "|"
            } elseif {$::Herramientas::bot($i)=="jpg"} {
                append filter (jpeg$) "|"
            } elseif {$::Herramientas::bot($i)=="mpg"} {
                append filter (mpeg$) "|"
            }
            append filter ($::Herramientas::bot($i)$) "|"
        }
    }
    regexp {(.*)(\|)} $filter nada filter

    return $filter
}

###############################################################################
# ReadSupportedLang
#    Reads the 'supported' file to know which languages this version supports
#    and puts then in supportedLang global variable
###############################################################################
proc ReadSupportedLang {} {
    global dirGetleft supportedLang

    if {[catch {open [file join $dirGetleft(languages) supported] r} handle]} {
        Dialogos::Dialogo .error -type ok -icon error -title Error          \
                -message $handle
        return
    }
    set encoding cp1252
    fconfigure $handle -encoding $encoding
    for {set supportedLang(langList) ""} {![eof $handle]} {} {
        set line [gets $handle]
        if {[regexp {^#|^ |^$} $line]} {
            continue
        }
        set key      [lindex $line 0]
        set encoding [lindex $line 2]
        if {[catch {encoding convertfrom $encoding [lindex $line 1]} language]} {
            continue
        }
        set supportedLang($key)         $encoding
        set supportedLang($language)    $key
        lappend supportedLang(langList) $language
    }
    set supportedLang(langList) [lsort $supportedLang(langList)]

    close $handle
    return
}

###############################################################################
# ReadLangFile
#    Reads one of the files with translations
#
# Parameter
#    The two letters code that identifies one language, i.e., 'es', 'en'
#
# Returns
#    - '0' all went well
#    - '1' there was an error
###############################################################################
proc ReadLangFile {sufix} {
    global dirGetleft supportedLang
    global labelMenus labelButtons  labelTitles labelMessages
    global labelDialogs labelMonths labelFrames
    global indexDialogs indexMenus indexButtons accMenus
    global labelCurlCodes confWizardLabels

    msgcat::mclocale $sufix
    msgcat::mcload   [file join $::tk_library msgs]
    package forget  ttk::dialog
    package require ttk::dialog

    set languageFile [file join $dirGetleft(languages) menus.$sufix]

    if {[catch {open $languageFile r} handle]} {
        Dialogos::Dialogo .error -icon error -type ok -title Error \
                -message "File not found"
        return 1
    }
    fconfigure $handle -encoding $supportedLang($sufix)

    for {set active ""} {![eof $handle]} {} {
        set line [gets $handle]
        if {[regexp {^#|^ |^$} $line]} {
            continue
        }
        if {![string compare $line "End"]} {
            set active ""
            continue
        }
        set key   [lindex $line 0]
        set value [lindex $line 1]
        # Just for menus
        set accelerator [lindex $line 2]

        set index [string first & $value]
        if {$index!=-1} {
            regsub {&} $value {} value
        }

        switch -exact -- $active {
            Menus {
                set indexMenus($key)       $index
                set labelMenus($key)       $value
                set accMenus($key)         $accelerator
            }
            Buttons {
                set indexButtons($key)     $index
                set labelButtons($key)     $value
            }
            Titles {
                set labelTitles($key)      $value
            }
            Messages {
                set labelMessages($key)    $value
            }
            Dialogs {
                set labelDialogs($key)     $value
                set indexDialogs($key)     $index
            }
            Months {
                set labelMonths($key)      $value
                set labelMonths($value)    $key
            }
            Frames {
                set labelFrames($key)      $value
            }
            curlCodes {
                set labelCurlCodes($key)   $value
            }
            confWizard {
                set confWizardLabels($key) $value
            }
            "" {
                set active $line
            }
        }
    }
    close $handle
    return 0
}

###############################################################################
# SetMacColours
#    Given a tablelist widget, this will set it to look more 'Mac like'.
#    Thanks to Kevin Walzer (http://www.codebykevin.com) for the values.
#
# Parameter
#    widget: The widget to adapt.
##############################################################################
proc SetMacColours {widget} {

    $widget configure -selectbackground   RoyalBlue1
    $widget configure -selectforeground   white
    $widget configure -stretch            all
    $widget configure -stripebackground   #EDF3FE
    $widget configure -relief             flat
    $widget configure -border             0
    $widget configure -showseparators     yes
    $widget configure -takefocus          0
    $widget configure -setfocus           1
    $widget configure -activestyle        none

    return
}

}

