###############################################################################
###############################################################################
##                          menuEntry
###############################################################################
###############################################################################
## Creates a context menu for an entry widget.
###############################################################################
###############################################################################
## (c) 2002-2008 Andres Garcia Garcia. fandom@telefonica.net
## You may distribute the contents of this file under the terms of the LGPL v2
###############################################################################
###############################################################################

namespace eval menuEntry {

# One of my ugly kludges
if {![info exists labelMenus(cut)]} {
    set ::labelMenus(cut)        Cut
    set ::labelMenus(copy)       Copy
    set ::labelMenus(paste)      Paste
    set ::labelMenus(clear)      Clear
    set ::labelMenus(selectAll) "Select all"

    set ::indexMenus(cut)        0
    set ::indexMenus(copy)       2
    set ::indexMenus(paste)      0
    set ::indexMenus(clear)      4
    set ::indexMenus(selectAll)  0
}

set ext $::getleftState(imgExt)

set iconImages(cut)     [image create photo                     \
        -file [file join "$dirGetleft(images)" editcut.$ext]]
set iconImages(copy)    [image create photo                     \
        -file [file join "$dirGetleft(images)" editcopy.$ext]]
set iconImages(paste)   [image create photo                     \
        -file [file join "$dirGetleft(images)" editpaste.$ext]]
set iconImages(clear)   [image create photo                     \
        -file [file join "$dirGetleft(images)" editclear.$ext]]

###############################################################################
# ReadSel
#    Reads the selection from the clipboard
#
# Returns
#    Whatever was in the clipboard
###############################################################################
proc ReadSel {} {
    global tcl_platform

    if {$tcl_platform(platform)=="unix"} {
        if {![catch {selection get} sel]} {
            return $sel
        }
    }

    if {[catch {selection get -selection CLIPBOARD} sel]} return
    return $sel
}

###############################################################################
# PopUpMenu
#    Manages the menu that pops up when you right-click in the entry.
#
# Parameters:
#    menu: Path of the menu.
#    entryPath: Path of the entry
#    X-Y: Coordinates where the menu should appear.
###############################################################################
proc PopUpMenu {menu entryPath X Y} {
    global indexMenus

    set sel [ReadSel]
    if {$sel==""} {
        $menu entryconfigure 2 -state disabled
    } else {
        $menu entryconfigure 2 -state normal
    }
    set sel [$entryPath selection present]
    if {$sel==1} {
        $menu entryconfigure 0 -state normal 
        $menu entryconfigure 1 -state normal
        $menu entryconfigure 3 -state normal
    } else {
        $menu entryconfigure 0 -state disabled
        $menu entryconfigure 1 -state disabled
        $menu entryconfigure 3 -state disabled
    }
    if {[string length [$entryPath get]]==0} {
        $menu entryconfigure 5 -state disabled
    } else {
        $menu entryconfigure 5 -state normal
    }

    $menu entryconfigure 0 -underline $indexMenus(cut)
    $menu entryconfigure 1 -underline $indexMenus(copy)
    $menu entryconfigure 2 -underline $indexMenus(paste)
    $menu entryconfigure 3 -underline $indexMenus(clear)
    $menu entryconfigure 5 -underline $indexMenus(selectAll)

    tk_popup $entryPath.menu $X $Y

    return
}

###############################################################################
# RightClick
#    The following mess makes sure the insert cursor is placed in the
#    character where you right-click, all the while keeping the selection
#    if you had one.
###############################################################################
proc RightClick {entryPath x y X Y} {
    variable sel1
    variable sel2

    set sel [$entryPath selection present]
    if {$sel==1} {
        set sel1 [$entryPath index sel.first]
        set sel2 [$entryPath index sel.last]
    } 
    focus $entryPath
#    event generate $entryPath <Button-1>        -x $x -y $y
#    event generate $entryPath <ButtonRelease-1> -x $x -y $y

    if {$sel==1} {
        $entryPath selection range $sel1 $sel2
    }
    menuEntry::PopUpMenu $entryPath.menu $entryPath $X $Y

    return
}

###############################################################################
# menuEntry
#    Links an entry with a menu to cut, copy, ...
#
# Parameter:
#    entryPath: The path to the entry.
###############################################################################
proc menuEntry {entryPath} {
    global labelMenus tcl_platform getleftOptions getleftState
    variable entries

    set menu [menu $entryPath.menu -tearoff 0 -bg $getleftOptions(menus)]
    if {[info exists getleftOptions(menuFont)]} {
        $menu configure -font $getleftOptions(menuFont)
    }
    $menu add command -label $labelMenus(cut)      -compound left  \
            -command "event generate $entryPath <<Cut>>"
    $menu add command -label $labelMenus(copy)     -compound left  \
            -command "event generate $entryPath <<Copy>>"
    $menu add command -label $labelMenus(paste)     -compound left \
            -command "event generate $entryPath <<Paste>>"
    $menu add command -label $labelMenus(clear)     -compound left \
            -command "event generate $entryPath <<Clear>>"
    $menu add separator
    $menu add command -label $labelMenus(selectAll)                \
            -command "$entryPath selection range 0 end"

    if {$getleftState(os) ne "mac"} {
        $menu entryconfigure 0 -image $menuEntry::iconImages(cut)
        $menu entryconfigure 1 -image $menuEntry::iconImages(copy)
        $menu entryconfigure 2 -image $menuEntry::iconImages(paste)
        $menu entryconfigure 3 -image $menuEntry::iconImages(clear)
    }

    bind $entryPath <<MiddleButton>> "event generate %W <<Paste>>"

    bind $entryPath <<ContextButton>> "menuEntry::RightClick %W %x %y %X %Y"
    if {$::getleftState(os)=="win"} {
        bind $entryPath <App>  "menuEntry::RightClick %W %x %y %X %Y"
    } else {
        bind $entryPath <Menu> "menuEntry::RightClick %W %x %y %X %Y"
    }

    set entries($entryPath) 0

    return
}
}
